@@ -35,7 +35,6 @@ static int quota_ctl(int cmd, int argc, char **argv)
 {
 	int ret = 0;
 	int fd;
-	int e;
 	char *path = argv[1];
 	struct btrfs_ioctl_quota_ctl_args args;
 	DIR *dirstream = NULL;
@@ -51,10 +50,9 @@ static int quota_ctl(int cmd, int argc, char **argv)
 		return 1;
 
 	ret = ioctl(fd, BTRFS_IOC_QUOTA_CTL, &args);
-	e = errno;
 	close_file_or_dir(fd, dirstream);
 	if (ret < 0) {
-		error("quota command failed: %s", strerror(e));
+		error("quota command failed: %m");
 		return 1;
 	}
 	return 0;
@@ -158,8 +156,7 @@ static int cmd_quota_rescan(int argc, char **argv)
 	if (ioctlnum == BTRFS_IOC_QUOTA_RESCAN_STATUS) {
 		close_file_or_dir(fd, dirstream);
 		if (ret < 0) {
-			error("could not obtain quota rescan status: %s",
-			      strerror(e));
+			error("could not obtain quota rescan status: %m");
 			return 1;
 		}
 		if (!args.flags)
@@ -174,7 +171,7 @@ static int cmd_quota_rescan(int argc, char **argv)
 		printf("quota rescan started\n");
 		fflush(stdout);
 	} else if (ret < 0 && (!wait_for_completion || e != EINPROGRESS)) {
-		error("quota rescan failed: %s", strerror(e));
+		error("quota rescan failed: %m");
 		close_file_or_dir(fd, dirstream);
 		return 1;
 	}
@@ -183,8 +180,7 @@ static int cmd_quota_rescan(int argc, char **argv)
 		ret = ioctl(fd, BTRFS_IOC_QUOTA_RESCAN_WAIT, &args);
 		e = errno;
 		if (ret < 0) {
-			error("quota rescan wait failed: %s",
-			      strerror(e));
+			error("quota rescan wait failed: %m");
 			close_file_or_dir(fd, dirstream);
 			return 1;
 		}
